"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from glean import errors, models, utils
from glean._hooks import HookContext
from glean.types import OptionalNullable, UNSET
from glean.utils import get_security_from_env
from typing import List, Mapping, Optional, Union


class Datasources(BaseSDK):
    def add(
        self,
        *,
        name: str,
        display_name: Optional[str] = None,
        datasource_category: Optional[
            models.DatasourceCategory
        ] = models.DatasourceCategory.UNCATEGORIZED,
        url_regex: Optional[str] = None,
        icon_url: Optional[str] = None,
        object_definitions: Optional[
            Union[List[models.ObjectDefinition], List[models.ObjectDefinitionTypedDict]]
        ] = None,
        suggestion_text: Optional[str] = None,
        home_url: Optional[str] = None,
        crawler_seed_urls: Optional[List[str]] = None,
        icon_dark_url: Optional[str] = None,
        hide_built_in_facets: Optional[List[models.HideBuiltInFacet]] = None,
        canonicalizing_url_regex: Optional[
            Union[
                List[models.CanonicalizingRegexType],
                List[models.CanonicalizingRegexTypeTypedDict],
            ]
        ] = None,
        canonicalizing_title_regex: Optional[
            Union[
                List[models.CanonicalizingRegexType],
                List[models.CanonicalizingRegexTypeTypedDict],
            ]
        ] = None,
        redlist_title_regex: Optional[str] = None,
        connector_type: Optional[models.CustomDatasourceConfigConnectorType] = None,
        quicklinks: Optional[
            Union[List[models.Quicklink], List[models.QuicklinkTypedDict]]
        ] = None,
        render_config_preset: Optional[str] = None,
        aliases: Optional[List[str]] = None,
        is_on_prem: Optional[bool] = None,
        trust_url_regex_for_view_activity: Optional[bool] = True,
        include_utm_source: Optional[bool] = None,
        strip_fragment_in_canonical_url: Optional[bool] = True,
        identity_datasource_name: Optional[str] = None,
        product_access_group: Optional[str] = None,
        is_user_referenced_by_email: Optional[bool] = None,
        is_entity_datasource: Optional[bool] = False,
        is_test_datasource: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Add or update datasource

        Add or update a custom datasource and its schema.

        :param name: Unique identifier of datasource instance to which this config applies.
        :param display_name: The user-friendly instance label to display. If omitted, falls back to the title-cased `name`.
        :param datasource_category: The type of this datasource. It is an important signal for relevance and must be specified and cannot be UNCATEGORIZED. Please refer to [this](https://developers.glean.com/docs/indexing_api_datasource_category/) for more details.
        :param url_regex: Regular expression that matches URLs of documents of the datasource instance. The behavior for multiple matches is non-deterministic. **Note: `urlRegex` is a required field for non-entity datasources, but not required if the datasource is used to push custom entities (ie. datasources where isEntityDatasource is false). Please add a regex as specific as possible to this datasource instance.**
        :param icon_url: The URL to an image to be displayed as an icon for this datasource instance. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs).
        :param object_definitions: The list of top-level `objectType`s for the datasource.
        :param suggestion_text: Example text for what to search for in this datasource
        :param home_url: The URL of the landing page for this datasource instance. Should point to the most useful page for users, not the company marketing page.
        :param crawler_seed_urls: This only applies to WEB_CRAWL and BROWSER_CRAWL datasources. Defines the seed URLs for crawling.
        :param icon_dark_url: The URL to an image to be displayed as an icon for this datasource instance in dark mode. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs).
        :param hide_built_in_facets: List of built-in facet types that should be hidden for the datasource.
        :param canonicalizing_url_regex: A list of regular expressions to apply to an arbitrary URL to transform it into a canonical URL for this datasource instance. Regexes are to be applied in the order specified in this list.
        :param canonicalizing_title_regex: A list of regular expressions to apply to an arbitrary title to transform it into a title that will be displayed in the search results
        :param redlist_title_regex: A regex that identifies titles that should not be indexed
        :param connector_type:
        :param quicklinks: List of actions for this datasource instance that will show up in autocomplete and app card, e.g. \"Create new issue\" for jira
        :param render_config_preset: The name of a render config to use for displaying results from this datasource. Any well known datasource name may be used to render the same as that source, e.g. `web` or `gdrive`. Please refer to [this](https://developers.glean.com/docs/rendering_search_results/) for more details
        :param aliases: Aliases that can be used as `app` operator-values.
        :param is_on_prem: Whether or not this datasource is hosted on-premise.
        :param trust_url_regex_for_view_activity: True if browser activity is able to report the correct URL for VIEW events. Set this to true if the URLs reported by Chrome are constant throughout each page load. Set this to false if the page has Javascript that modifies the URL during or after the load.
        :param include_utm_source: If true, a utm_source query param will be added to outbound links to this datasource within Glean.
        :param strip_fragment_in_canonical_url: If true, the fragment part of the URL will be stripped when converting to a canonical url.
        :param identity_datasource_name: If the datasource uses another datasource for identity info, then the name of the datasource. The identity datasource must exist already.
        :param product_access_group: If the datasource uses a specific product access group, then the name of that group.
        :param is_user_referenced_by_email: whether email is used to reference users in document ACLs and in group memberships.
        :param is_entity_datasource: True if this datasource is used to push custom entities.
        :param is_test_datasource: True if this datasource will be used for testing purpose only. Documents from such a datasource wouldn't have any effect on search rankings.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomDatasourceConfig(
            name=name,
            display_name=display_name,
            datasource_category=datasource_category,
            url_regex=url_regex,
            icon_url=icon_url,
            object_definitions=utils.get_pydantic_model(
                object_definitions, Optional[List[models.ObjectDefinition]]
            ),
            suggestion_text=suggestion_text,
            home_url=home_url,
            crawler_seed_urls=crawler_seed_urls,
            icon_dark_url=icon_dark_url,
            hide_built_in_facets=hide_built_in_facets,
            canonicalizing_url_regex=utils.get_pydantic_model(
                canonicalizing_url_regex, Optional[List[models.CanonicalizingRegexType]]
            ),
            canonicalizing_title_regex=utils.get_pydantic_model(
                canonicalizing_title_regex,
                Optional[List[models.CanonicalizingRegexType]],
            ),
            redlist_title_regex=redlist_title_regex,
            connector_type=connector_type,
            quicklinks=utils.get_pydantic_model(
                quicklinks, Optional[List[models.Quicklink]]
            ),
            render_config_preset=render_config_preset,
            aliases=aliases,
            is_on_prem=is_on_prem,
            trust_url_regex_for_view_activity=trust_url_regex_for_view_activity,
            include_utm_source=include_utm_source,
            strip_fragment_in_canonical_url=strip_fragment_in_canonical_url,
            identity_datasource_name=identity_datasource_name,
            product_access_group=product_access_group,
            is_user_referenced_by_email=is_user_referenced_by_email,
            is_entity_datasource=is_entity_datasource,
            is_test_datasource=is_test_datasource,
        )

        req = self._build_request(
            method="POST",
            path="/api/index/v1/adddatasource",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomDatasourceConfig
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post_/api/index/v1/adddatasource",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return
        if utils.match_response(http_res, ["400", "401", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.GleanError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def add_async(
        self,
        *,
        name: str,
        display_name: Optional[str] = None,
        datasource_category: Optional[
            models.DatasourceCategory
        ] = models.DatasourceCategory.UNCATEGORIZED,
        url_regex: Optional[str] = None,
        icon_url: Optional[str] = None,
        object_definitions: Optional[
            Union[List[models.ObjectDefinition], List[models.ObjectDefinitionTypedDict]]
        ] = None,
        suggestion_text: Optional[str] = None,
        home_url: Optional[str] = None,
        crawler_seed_urls: Optional[List[str]] = None,
        icon_dark_url: Optional[str] = None,
        hide_built_in_facets: Optional[List[models.HideBuiltInFacet]] = None,
        canonicalizing_url_regex: Optional[
            Union[
                List[models.CanonicalizingRegexType],
                List[models.CanonicalizingRegexTypeTypedDict],
            ]
        ] = None,
        canonicalizing_title_regex: Optional[
            Union[
                List[models.CanonicalizingRegexType],
                List[models.CanonicalizingRegexTypeTypedDict],
            ]
        ] = None,
        redlist_title_regex: Optional[str] = None,
        connector_type: Optional[models.CustomDatasourceConfigConnectorType] = None,
        quicklinks: Optional[
            Union[List[models.Quicklink], List[models.QuicklinkTypedDict]]
        ] = None,
        render_config_preset: Optional[str] = None,
        aliases: Optional[List[str]] = None,
        is_on_prem: Optional[bool] = None,
        trust_url_regex_for_view_activity: Optional[bool] = True,
        include_utm_source: Optional[bool] = None,
        strip_fragment_in_canonical_url: Optional[bool] = True,
        identity_datasource_name: Optional[str] = None,
        product_access_group: Optional[str] = None,
        is_user_referenced_by_email: Optional[bool] = None,
        is_entity_datasource: Optional[bool] = False,
        is_test_datasource: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ):
        r"""Add or update datasource

        Add or update a custom datasource and its schema.

        :param name: Unique identifier of datasource instance to which this config applies.
        :param display_name: The user-friendly instance label to display. If omitted, falls back to the title-cased `name`.
        :param datasource_category: The type of this datasource. It is an important signal for relevance and must be specified and cannot be UNCATEGORIZED. Please refer to [this](https://developers.glean.com/docs/indexing_api_datasource_category/) for more details.
        :param url_regex: Regular expression that matches URLs of documents of the datasource instance. The behavior for multiple matches is non-deterministic. **Note: `urlRegex` is a required field for non-entity datasources, but not required if the datasource is used to push custom entities (ie. datasources where isEntityDatasource is false). Please add a regex as specific as possible to this datasource instance.**
        :param icon_url: The URL to an image to be displayed as an icon for this datasource instance. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs).
        :param object_definitions: The list of top-level `objectType`s for the datasource.
        :param suggestion_text: Example text for what to search for in this datasource
        :param home_url: The URL of the landing page for this datasource instance. Should point to the most useful page for users, not the company marketing page.
        :param crawler_seed_urls: This only applies to WEB_CRAWL and BROWSER_CRAWL datasources. Defines the seed URLs for crawling.
        :param icon_dark_url: The URL to an image to be displayed as an icon for this datasource instance in dark mode. Must have a transparency mask. SVG are recommended over PNG. Public, scio-authenticated and Base64 encoded data URLs are all valid (but not third-party-authenticated URLs).
        :param hide_built_in_facets: List of built-in facet types that should be hidden for the datasource.
        :param canonicalizing_url_regex: A list of regular expressions to apply to an arbitrary URL to transform it into a canonical URL for this datasource instance. Regexes are to be applied in the order specified in this list.
        :param canonicalizing_title_regex: A list of regular expressions to apply to an arbitrary title to transform it into a title that will be displayed in the search results
        :param redlist_title_regex: A regex that identifies titles that should not be indexed
        :param connector_type:
        :param quicklinks: List of actions for this datasource instance that will show up in autocomplete and app card, e.g. \"Create new issue\" for jira
        :param render_config_preset: The name of a render config to use for displaying results from this datasource. Any well known datasource name may be used to render the same as that source, e.g. `web` or `gdrive`. Please refer to [this](https://developers.glean.com/docs/rendering_search_results/) for more details
        :param aliases: Aliases that can be used as `app` operator-values.
        :param is_on_prem: Whether or not this datasource is hosted on-premise.
        :param trust_url_regex_for_view_activity: True if browser activity is able to report the correct URL for VIEW events. Set this to true if the URLs reported by Chrome are constant throughout each page load. Set this to false if the page has Javascript that modifies the URL during or after the load.
        :param include_utm_source: If true, a utm_source query param will be added to outbound links to this datasource within Glean.
        :param strip_fragment_in_canonical_url: If true, the fragment part of the URL will be stripped when converting to a canonical url.
        :param identity_datasource_name: If the datasource uses another datasource for identity info, then the name of the datasource. The identity datasource must exist already.
        :param product_access_group: If the datasource uses a specific product access group, then the name of that group.
        :param is_user_referenced_by_email: whether email is used to reference users in document ACLs and in group memberships.
        :param is_entity_datasource: True if this datasource is used to push custom entities.
        :param is_test_datasource: True if this datasource will be used for testing purpose only. Documents from such a datasource wouldn't have any effect on search rankings.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CustomDatasourceConfig(
            name=name,
            display_name=display_name,
            datasource_category=datasource_category,
            url_regex=url_regex,
            icon_url=icon_url,
            object_definitions=utils.get_pydantic_model(
                object_definitions, Optional[List[models.ObjectDefinition]]
            ),
            suggestion_text=suggestion_text,
            home_url=home_url,
            crawler_seed_urls=crawler_seed_urls,
            icon_dark_url=icon_dark_url,
            hide_built_in_facets=hide_built_in_facets,
            canonicalizing_url_regex=utils.get_pydantic_model(
                canonicalizing_url_regex, Optional[List[models.CanonicalizingRegexType]]
            ),
            canonicalizing_title_regex=utils.get_pydantic_model(
                canonicalizing_title_regex,
                Optional[List[models.CanonicalizingRegexType]],
            ),
            redlist_title_regex=redlist_title_regex,
            connector_type=connector_type,
            quicklinks=utils.get_pydantic_model(
                quicklinks, Optional[List[models.Quicklink]]
            ),
            render_config_preset=render_config_preset,
            aliases=aliases,
            is_on_prem=is_on_prem,
            trust_url_regex_for_view_activity=trust_url_regex_for_view_activity,
            include_utm_source=include_utm_source,
            strip_fragment_in_canonical_url=strip_fragment_in_canonical_url,
            identity_datasource_name=identity_datasource_name,
            product_access_group=product_access_group,
            is_user_referenced_by_email=is_user_referenced_by_email,
            is_entity_datasource=is_entity_datasource,
            is_test_datasource=is_test_datasource,
        )

        req = self._build_request_async(
            method="POST",
            path="/api/index/v1/adddatasource",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CustomDatasourceConfig
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post_/api/index/v1/adddatasource",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "*"):
            return
        if utils.match_response(http_res, ["400", "401", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.GleanError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def retrieve_config(
        self,
        *,
        datasource: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomDatasourceConfig:
        r"""Get datasource config

        Fetches the datasource config for the specified custom datasource.

        :param datasource: Datasource name for which config is needed.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetDatasourceConfigRequest(
            datasource=datasource,
        )

        req = self._build_request(
            method="POST",
            path="/api/index/v1/getdatasourceconfig",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.GetDatasourceConfigRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post_/api/index/v1/getdatasourceconfig",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "409", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CustomDatasourceConfig)
        if utils.match_response(http_res, ["400", "401", "409", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.GleanError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def retrieve_config_async(
        self,
        *,
        datasource: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CustomDatasourceConfig:
        r"""Get datasource config

        Fetches the datasource config for the specified custom datasource.

        :param datasource: Datasource name for which config is needed.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetDatasourceConfigRequest(
            datasource=datasource,
        )

        req = self._build_request_async(
            method="POST",
            path="/api/index/v1/getdatasourceconfig",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.GetDatasourceConfigRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post_/api/index/v1/getdatasourceconfig",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "409", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.CustomDatasourceConfig)
        if utils.match_response(http_res, ["400", "401", "409", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.GleanError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.GleanError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )
